package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.Statement;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class ThrowStatement extends Statement {
  public Expression exception;
  
  public TypeBinding exceptionType;
  
  public ThrowStatement(Expression exception, int sourceStart, int sourceEnd) {
    this.exception = exception;
    this.sourceStart = sourceStart;
    this.sourceEnd = sourceEnd;
  }
  
  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
    this.exception.analyseCode(currentScope, flowContext, flowInfo);
    this.exception.checkNPE(currentScope, flowContext, flowInfo);
    flowContext.checkExceptionHandlers(this.exceptionType, (ASTNode)this, flowInfo, currentScope);
    currentScope.checkUnclosedCloseables(flowInfo, flowContext, (ASTNode)this, currentScope);
    flowContext.recordAbruptExit();
    return (FlowInfo)FlowInfo.DEAD_END;
  }
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream) {
    if ((this.bits & Integer.MIN_VALUE) == 0)
      return; 
    int pc = codeStream.position;
    this.exception.generateCode(currentScope, codeStream, true);
    codeStream.athrow();
    codeStream.recordPositionsFrom(pc, this.sourceStart);
  }
  
  public StringBuffer printStatement(int indent, StringBuffer output) {
    printIndent(indent, output).append("throw ");
    this.exception.printExpression(0, output);
    return output.append(';');
  }
  
  public void resolve(BlockScope scope) {
    this.exceptionType = this.exception.resolveType(scope);
    if (this.exceptionType != null && this.exceptionType.isValidBinding()) {
      if (this.exceptionType == TypeBinding.NULL) {
        if ((scope.compilerOptions()).complianceLevel <= 3080192L)
          scope.problemReporter().cannotThrowNull((ASTNode)this.exception); 
      } else if (this.exceptionType.findSuperTypeOriginatingFrom(21, true) == null) {
        scope.problemReporter().cannotThrowType((ASTNode)this.exception, this.exceptionType);
      } 
      this.exception.computeConversion((Scope)scope, this.exceptionType, this.exceptionType);
    } 
  }
  
  public void traverse(ASTVisitor visitor, BlockScope blockScope) {
    if (visitor.visit(this, blockScope))
      this.exception.traverse(visitor, blockScope); 
    visitor.endVisit(this, blockScope);
  }
  
  public boolean doesNotCompleteNormally() {
    return true;
  }
}
